from django.contrib import admin
from eventy.models import *

class EventTimeAdmin(admin.ModelAdmin):
    list_display = ('event', 'start', 'end', 'place', )
    search_fields = ('event',)
    list_filter = ( 'place', 'event', )

admin.site.register(EventTime, EventTimeAdmin)

class EventAdmin(admin.ModelAdmin):
    list_display = ('title', 'calendar', 'submitted_by',)
    search_fields = ('title','calendar',)
    list_filter = ('calendar', )

admin.site.register(Event, EventAdmin)
    
admin.site.register(Place)
admin.site.register(Calendar)



from django.conf.urls.defaults import *
from django.views.generic import ListView, DetailView
from eventy.models import Event, EventTime, Calendar, Place
from eventy.views import EventDetailView, EventDayView, EventMonthView, EventYearView, EventListView

urlpatterns = patterns('',
    url(r'^(?P<year>\d{4})/(?P<month>\w{3})/(?P<day>\d{1,2})/$', view=EventDayView.as_view(),    name='ev-events-day'),
    url(r'^(?P<year>\d{4})/(?P<month>\w{3})/$',                  view=EventMonthView.as_view(),  name='ev-events-month'),
    url(r'^(?P<year>\d{4})/$',                                   view=EventYearView.as_view(),   name='ev-events-year'),

    url(r'^calendars/(?P<cal_slug>[-\w]+)/(?P<year>\d{4})/$',                  view=EventYearView.as_view(),   name='ev-calendar-year'),
    url(r'^calendars/(?P<cal_slug>[-\w]+)/(?P<year>\d{4})/(?P<month>\w{3})/$', view=EventMonthView.as_view(),  name='ev-calendar-month'),
    url(r'^calendars/(?P<cal_slug>[-\w]+)/$',                                  view=EventListView.as_view(),   name='ev-calendar-detail'),
    url(r'^calendars/$',                                                       view=ListView.as_view(model=Calendar), name='ev-calendar-list'),

    url(r'^places/$', view=ListView.as_view(model=Place), name='place-list'),
    url(r'^places/(?P<slug>[-\w]+)/$', view=DetailView.as_view(model=Place), name='place-detail'),

    url(r'^(?P<slug>[-\w]+)/(?P<year>\d{4})/(?P<month>\w{3})/(?P<day>\d{1,2})/$', view=EventDetailView.as_view(), name='ev-event-detail'),
    url(r'^(?P<slug>[-\w]+)/(?P<year>\d{4})/(?P<month>\w{3})/$',                  view=EventMonthView.as_view(),  name='ev-event-month'),
    url(r'^(?P<slug>[-\w]+)/(?P<year>\d{4})/$',                                   view=EventYearView.as_view(),   name='ev-event-year'),
    url(r'^(?P<slug>[-\w]+)/$',                                                   view=DetailView.as_view(model=Event), name='ev-events-info'),
    url(r'^$', view=EventListView.as_view(), name='event-list'),
)



import calendar
import logging
from datetime import date
from itertools import groupby

from django.utils.html import conditional_escape as esc

class EventCalendar(calendar.HTMLCalendar):
    def __init__(self, events, size='large'):
        super(EventCalendar, self).__init__()
        self.events = self.group_by_day(events)
        self.size = size
    
    def formatday(self, day, weekday):
        if day != 0:
            cssclass = self.cssclasses[weekday]
            if date.today() == date(self.year, self.month, day):
                cssclass += ' today'
            if day in self.events:
                cssclass += ' filled'
                if self.size == 'large':
                    body = ['<ul>']
                    for event in self.events[day]:
                        body.append('<li>')
                        body.append('<a href="%s">' % event.get_absolute_url())
                        body.append(esc(event.event.title))
                        body.append('</a></li>')
                        body.append('</ul>')
                    return self.day_cell(cssclass, '%d %s' % (day, ''.join(body)))
                else:
                    for event in self.events[day]:
                        body = ['<a href="%s"><div class="%s">%d</div></a>' % (event.get_absolute_url(), event.event.calendar.slug, day)]
                    return self.day_cell(cssclass, '%s' % ''.join(body))
            return self.day_cell(cssclass, '%d %s' % (day, ''.join('<div class="noevents"></div>')))
        return self.day_cell('noday', '&nbsp;')

    def formatmonth(self, year, month, **kwargs):
        self.year, self.month = year, month
        return super(EventCalendar, self).formatmonth(year, month, 100)

    def formatyear(self, year):
        self.year = year
        return super(EventCalendar, self).formatyear(year)

    def group_by_day(self, events):
        field = lambda event: event.start.day
        return dict([(day, list(items)) for day, items in groupby(events, field)])

    def day_cell(self, cssclass, body):
        return '<td class="%s">%s</td>' % (cssclass, body)



from datetime import datetime, timedelta
import logging
from django.db import models
from django.conf import settings
from django import template
from django.core.urlresolvers import reverse
from django.utils.dateformat import format
from eventy.models import Calendar, EventTime
from eventy.utils import EventCalendar

register = template.Library()

def do_get_next_events(parser, token):
    ''' Gets a number of upcoming events for a given calendar slug and loads a template variable.

        Example usage:

        {% get_next_events 5 from vacation as events %} 
        {% get_next_events 5 from all as events %} 
        {% get_next_events 5 from all but services as events %} 
        {% get_next_events 5 from all but services,office as events %} 

    '''

    args = token.split_contents()
    argc = len(args)

    try:
        assert argc in (6, 8)
    except AssertionError:
        raise template.TemplateSyntaxError('Invalid get_committee_groups syntax.')
    # determine what parameters to use
    limit = cal_slug = excluded_cals = varname = None
    if argc == 6: t, limit, f, cal_slug, a, varname = args
    elif argc == 8: t, limit, f, cal_slug, b, excluded_cals, a, varname = args
    return GetNextEventsNode(limit=limit, calendar_slug=cal_slug, excluded_cals=excluded_cals, varname=varname)

class GetNextEventsNode(template.Node):
    def __init__(self, limit, calendar_slug, excluded_cals, varname):
        self.limit, self.varname, self.calendar_slug, = int(limit), varname, calendar_slug
        try:
            self.excluded_cals = excluded_cals.split(',')
        except:
            self.excluded_cals = []

    def render(self, context):
        if self.calendar_slug == 'all':
            events = EventTime.upcoming_objects.all().order_by('start')
        else:
            events = EventTime.upcoming_objects.filter(event__calendar__slug=self.calendar_slug).order_by('start')

        if self.excluded_cals:
            for cal_slug in self.excluded_cals:
                events = events.exclude(event__calendar__slug=cal_slug)

        logging.debug('Fetching %s events from calendar %s excluding %s.' % (self.limit, self.calendar_slug, self.excluded_cals))
        if self.limit == 1:
            try:
                context[self.varname] = events[0]
            except:
                context[self.varname] = None
        else:
            try:
                context[self.varname] = events[:self.limit]
            except:
                context[self.varname] = None
        return ''

register.tag("get_next_events", do_get_next_events)

@register.inclusion_tag('eventy/_month_table.html')
def show_month(events, date, size="large"):
    if date == '':
        month_table = EventCalendar(events, size).formatmonth(datetime.now().year, datetime.now().month)
    else:
        month_table = EventCalendar(events, size).formatmonth(date.year, date.month)
    return {'month_table':month_table, 'size':size }

@register.inclusion_tag('eventy/_year_table.html')
def show_year(events, date, size="small"):
    year_table = EventCalendar(events, size).formatyear(int(date))
    return {'year_table':year_table }



from datetime import datetime
from django.db.models import Manager
from django.db.models import Q

class UpcomingManager(Manager):
    """Returns upcoming events."""

    def get_query_set(self):
        return super(UpcomingManager, self).get_query_set().filter(start__gte=datetime.now())



from datetime import datetime

from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.utils.translation import ugettext_lazy as _
from django.db.models import permalink
from django.contrib.auth.models import User
from django.contrib.localflavor.us.models import USStateField

from onec_utils.models import USAddressPhoneMixin
from django_extensions.db.models import TimeStampedModel, TitleSlugDescriptionModel

from eventy.managers import UpcomingManager

class Place(TitleSlugDescriptionModel, TimeStampedModel, USAddressPhoneMixin):

    class Meta:
        verbose_name = _('place')
        verbose_name_plural = _('places')

    def __unicode__(self):
        return self.title

class Calendar(TitleSlugDescriptionModel, TimeStampedModel):
    """Calendar model."""

    class Meta:
        verbose_name = _('calendar')
        verbose_name_plural=_('calendars')

    def __unicode__(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        return ('event-calendar-detail', None, { 'cal_slug': self.slug, })

class Event(TitleSlugDescriptionModel, TimeStampedModel):
    """Event model"""
    calendar=models.ForeignKey(Calendar, blank=True, null=True)
    submitted_by = models.ForeignKey(User, blank=True, null=True)

    class Meta:
        verbose_name = _('event')
        verbose_name_plural = _('events')

    def __init__(self, *args, **kwargs):
        super (Event, self).__init__(*args, **kwargs)
        self._upcoming = None
        self._past = None

    @property
    def upcoming_event_times(self):
        if not self._upcoming:
            try:
                times = self.event_times.all().filter(start__gte=datetime.now().date)
            except(Event.DoesNotExist, IndexError):
                times = None
            self._upcoming = times
        return self._upcoming


    @property
    def past_event_times(self):
        if not self._past:
            try:
                times = self.event_times.all().filter(start__lte=datetime.now().date)
            except(Event.DoesNotExist, IndexError):
                times = None
            self._past= times
        return self._past

    def __unicode__(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        return ('event-info', None, { 'slug': self.event.slug, })


class EventTime(models.Model):
    """EventTime model
    
    This model does most of the heavy lifting as to whether """
    event = models.ForeignKey(Event, related_name='event_times')
    start = models.DateTimeField()
    end = models.DateTimeField(blank=True, null=True)
    place = models.ForeignKey(Place, blank=True, null=True)
    place_string = models.CharField(_('One-off place'), max_length=200, blank=True)
    is_all_day = models.BooleanField(default=False)
    notes = models.TextField(_('Notes'), blank=True, null=True)
    slug = models.SlugField(_('Event slug'), blank=True, editable=False)

    objects = models.Manager()
    upcoming_objects = UpcomingManager()

    class Meta:
        verbose_name = _('event time')
        verbose_name_plural = _('event times')
        get_latest_by = 'start'

    def __init__(self, *args, **kwargs):
        super (EventTime, self).__init__(*args, **kwargs)
        self._next = None
        self._previous = None

    @property
    def is_past(self):
        NOW = datetime.now().date
        if self.start < NOW:
            return True
        return False

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = self.event.slug

        if self.place and not self.place_string:
            self.place_string = self.place.__unicode__()
        super(EventTime, self).save(*args, **kwargs)

    def __unicode__(self):
        return u'%s' % self.event.title

    @models.permalink
    def get_absolute_url(self):
        return ('ev-event-detail', None, {
            'year': self.start.year,
            'month': self.start.strftime('%b').lower(),
            'day': self.start.day,
            'slug': self.event.slug,
        })

    def get_next_event_time(self):
        """Determines the next event time"""

        if not self._next:
            try:
                qs = EventTime.objects.filter(event=self.event).exclude(id__exact=self.id)
                event_time= qs.filter(start__gte=self.start).order_by('start')[0]
            except (EventTime.DoesNotExist, IndexError):
                event_time = None
            self._next = event_time

        return self._next

    def get_previous_event_time(self):
        """Determines the previous event time"""

        if not self._previous:
            try:
                qs = EventTime.objects.all().exclude(id__exact=self.id)
                event_time= qs.filter(start__lte=self.start).order_by('-start')[0]
            except (EventTime.DoesNotExist, IndexError):
                event_time = None
            self._previous = event_time

        return self._previous

class RelatedEvent(models.Model):
    eventtime = models.ForeignKey(EventTime)
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    def __unicode__(self):
        return self.eventtime.event.__unicode__()



from dateutil.relativedelta import relativedelta
from datetime import datetime
import logging

from django.template.context import RequestContext
from django.views.generic import ListView, DetailView
from django.views.generic.dates import DateDetailView, YearArchiveView, MonthArchiveView, DayArchiveView
from django.shortcuts import render_to_response, get_object_or_404

from eventy.models import EventTime, Calendar, Event
from eventy.utils import EventCalendar 

class EventListView(ListView):
    ''' EventListView
    Extend ListView with EventTime objects and load a calendar if the cal_slug is present.
    '''
    context_object_name = "event_list"
    date_field = 'start'

    def get_queryset(self):
        try:
            cal_slug = self.kwargs['cal_slug']
            self.calendar = get_object_or_404(Calendar, slug=cal_slug)
        except:
            self.calendar = None
        return EventTime.upcoming_objects.all()

    def get_context_data(self, **kwargs):
        context = super(EventListView, self).get_context_data(**kwargs)
        context['month'] = EventCalendar(self.get_queryset()).formatmonth(datetime.now().year, datetime.now().month)
        if self.calendar:
            context['calendar'] = self.calendar
        return context

class EventDetailView(DateDetailView):
    context_object_name = "event"
    queryset=EventTime.upcoming_objects.all()
    allow_future = True
    date_field = 'start'

class EventYearView(YearArchiveView):
    context_object_name = "event_list"
    allow_future = True
    date_field = 'start'
    allow_empty = True

    def get_queryset(self):
        try:
            if self.kwargs['cal_slug']:
                return EventTime.upcoming_objects.filter(event__calendar__slug=self.kwargs['cal_slug'])
        except:
            pass

        try:
            if self.kwargs['slug']:
                return EventTime.upcoming_objects.filter(event__slug=self.kwargs['slug'])
        except:
            logging.debug('Returning all upcoming events...')
            return EventTime.upcoming_objects.all()

    def get_context_data(self, **kwargs):
        context = super(EventYearView, self).get_context_data(**kwargs)
        try:
            context['next_date'] = context['date_list'][0] + relativedelta(years=1)
            context['prev_date'] = context['date_list'][0] - relativedelta(years=1)
        except:
            pass
        return context

class EventMonthView(MonthArchiveView):
    context_object_name = "event_list"
    allow_future = True
    date_field = 'start'
    allow_empty = True

    def get_queryset(self):
        try:
            if self.kwargs['cal_slug']:
                return EventTime.upcoming_objects.filter(event__calendar__slug=self.kwargs['cal_slug'])
        except:
            pass

        try:
            if self.kwargs['slug']:
                return EventTime.upcoming_objects.filter(event__slug=self.kwargs['slug'])
        except:
            return EventTime.upcoming_objects.all()

    def get_context_data(self, **kwargs):
        context = super(EventMonthView, self).get_context_data(**kwargs)
        try:
            context['next_date'] = next_date = context['date_list'][0] + relativedelta(months=1)
            context['prev_date'] = prev_date = context['date_list'][0] - relativedelta(months=1)
            context['next_month_str'] = next_date.strftime('%b').lower()
            context['prev_month_str'] = prev_date.strftime('%b').lower()
        except:
            pass
        logging.debug('Month is: %s' % context)
        return context
    
class EventDayView(DayArchiveView):
    context_object_name = "event_list"
    queryset=EventTime.upcoming_objects.all()
    allow_future = True
    date_field = 'start'
    allow_empty = True
